@@ -41,6 +41,92 @@ test_that("cars", {
 
 })
 
+test_that("cars - color", {
+  library(tablespan)
+  library(testthat)
+  library(dplyr)
+  library(openxlsx)
+
+  comp_file_dir <- paste0(testthat::test_path(), "/xlsx_files/")
+
+  summarized_table <- mtcars |>
+    group_by(cyl, vs) |>
+    summarise(N = n(),
+              mean_hp = mean(hp),
+              sd_hp = sd(hp),
+              mean_wt = mean(wt),
+              sd_wt = sd(wt))
+
+  tbl <- tablespan(data = summarized_table,
+                   formula = Cylinder:cyl + Engine:vs ~
+                     N +
+                     (`Horse Power` = Mean:mean_hp + SD:sd_hp) +
+                     (`Weight` = Mean:mean_wt + SD:sd_wt),
+                   title = "Motor Trend Car Road Tests",
+                   subtitle = "A table created with tablespan",
+                   footnote = "Data from the infamous mtcars data set.")
+
+  wb <- as_excel(tbl = tbl, styles = style_color(primary_color = "#987349"))
+
+  # Compare just the data
+  xlsx_compare <- openxlsx::read.xlsx(xlsxFile = paste0(comp_file_dir, "cars_987349.xlsx"))
+  testthat::expect_true(all.equal(openxlsx::read.xlsx(wb), xlsx_compare))
+
+  # to compare workbooks, we have to write and reload the xlsx file
+  tmp_dir <- tempdir()
+  openxlsx::saveWorkbook(wb,
+                         file = paste0(tmp_dir, "/cars_test.xlsx"),
+                         overwrite = TRUE)
+
+  wb <- openxlsx::loadWorkbook(file = paste0(tmp_dir, "/cars_test.xlsx"))
+  wb_compare <- openxlsx::loadWorkbook(file = paste0(comp_file_dir, "cars.xlsx"))
+  testthat::expect_true(all.equal(wb$worksheets,  wb_compare$worksheets))
+
+})
+
+test_that("cars - color - 2", {
+  library(tablespan)
+  library(testthat)
+  library(dplyr)
+  library(openxlsx)
+
+  comp_file_dir <- paste0(testthat::test_path(), "/xlsx_files/")
+
+  summarized_table <- mtcars |>
+    group_by(cyl, vs) |>
+    summarise(N = n(),
+              mean_hp = mean(hp),
+              sd_hp = sd(hp),
+              mean_wt = mean(wt),
+              sd_wt = sd(wt))
+
+  tbl <- tablespan(data = summarized_table,
+                   formula = Cylinder:cyl + Engine:vs ~
+                     N +
+                     (`Horse Power` = Mean:mean_hp + SD:sd_hp) +
+                     (`Weight` = Mean:mean_wt + SD:sd_wt),
+                   title = "Motor Trend Car Road Tests",
+                   subtitle = "A table created with tablespan",
+                   footnote = "Data from the infamous mtcars data set.")
+
+  wb <- as_excel(tbl = tbl, styles = style_color(primary_color = "#893485"))
+
+  # Compare just the data
+  xlsx_compare <- openxlsx::read.xlsx(xlsxFile = paste0(comp_file_dir, "cars_893485.xlsx"))
+  testthat::expect_true(all.equal(openxlsx::read.xlsx(wb), xlsx_compare))
+
+  # to compare workbooks, we have to write and reload the xlsx file
+  tmp_dir <- tempdir()
+  openxlsx::saveWorkbook(wb,
+                         file = paste0(tmp_dir, "/cars_test.xlsx"),
+                         overwrite = TRUE)
+
+  wb <- openxlsx::loadWorkbook(file = paste0(tmp_dir, "/cars_test.xlsx"))
+  wb_compare <- openxlsx::loadWorkbook(file = paste0(comp_file_dir, "cars.xlsx"))
+  testthat::expect_true(all.equal(wb$worksheets,  wb_compare$worksheets))
+
+})
+
 test_that("cars-offset", {
   library(tablespan)
   library(testthat)
@@ -395,3 +481,48 @@ test_that("cars-table_with_higher_spanner_left_right", {
   testthat::expect_true(all.equal(wb$worksheets,  wb_compare$worksheets))
 
 })
+
+test_that("cars - missing rownames", {
+  library(tablespan)
+  library(testthat)
+  library(dplyr)
+  library(openxlsx)
+
+  comp_file_dir <- paste0(testthat::test_path(), "/xlsx_files/")
+
+  summarized_table <- mtcars |>
+    group_by(cyl, vs) |>
+    summarise(N = n(),
+              mean_hp = mean(hp),
+              sd_hp = sd(hp),
+              mean_wt = mean(wt),
+              sd_wt = sd(wt))
+
+  summarized_table[1,1] <- NA
+  summarized_table[2,1] <- NA
+
+  tbl <- tablespan(data = summarized_table,
+                   formula = Cylinder:cyl + Engine:vs ~
+                     N +
+                     (`Horse Power` = Mean:mean_hp + SD:sd_hp) +
+                     (`Weight` = Mean:mean_wt + SD:sd_wt),
+                   title = "Motor Trend Car Road Tests",
+                   subtitle = "A table created with tablespan",
+                   footnote = "Data from the infamous mtcars data set.")
+
+  wb <- as_excel(tbl = tbl)
+
+  # Compare just the data
+  xlsx_compare <- openxlsx::read.xlsx(xlsxFile = paste0(comp_file_dir, "cars_missing_rownames.xlsx"))
+  testthat::expect_true(all.equal(openxlsx::read.xlsx(wb), xlsx_compare))
+
+  # to compare workbooks, we have to write and reload the xlsx file
+  tmp_dir <- tempdir()
+  openxlsx::saveWorkbook(wb,
+                         file = paste0(tmp_dir, "/cars_test.xlsx"),
+                         overwrite = TRUE)
+
+  wb <- openxlsx::loadWorkbook(file = paste0(tmp_dir, "/cars_test.xlsx"))
+  wb_compare <- openxlsx::loadWorkbook(file = paste0(comp_file_dir, "cars_missing_rownames.xlsx"))
+  testthat::expect_true(all.equal(wb$worksheets,  wb_compare$worksheets))
+})
